#include "exception.h"

int evalException(const int exception, const EXCEPTION_POINTERS* pinfo)
{
  char *msg;
  // Pass on most exceptions
  switch(exception)
  {
    case EXCEPTION_ACCESS_VIOLATION:         msg = "EXCEPTION_ACCESS_VIOLATION"; break;
    case EXCEPTION_DATATYPE_MISALIGNMENT:    msg = "EXCEPTION_DATATYPE_MISALIGNMENT"; break;
    case EXCEPTION_BREAKPOINT:               msg = "EXCEPTION_BREAKPOINT"; break;
    case EXCEPTION_SINGLE_STEP:              msg = "EXCEPTION_SINGLE_STEP"; break;
    case EXCEPTION_ARRAY_BOUNDS_EXCEEDED:    msg = "EXCEPTION_ARRAY_BOUNDS_EXCEEDED"; break;
    case EXCEPTION_FLT_DENORMAL_OPERAND:     msg = "EXCEPTION_FLT_DENORMAL_OPERAND"; break;
    case EXCEPTION_FLT_DIVIDE_BY_ZERO:       msg = "EXCEPTION_FLT_DIVIDE_BY_ZERO"; break;
    case EXCEPTION_FLT_INEXACT_RESULT:       msg = "EXCEPTION_FLT_INEXACT_RESULT"; break;
    case EXCEPTION_FLT_INVALID_OPERATION:    msg = "EXCEPTION_FLT_INVALID_OPERATION"; break;
    case EXCEPTION_FLT_OVERFLOW:             msg = "EXCEPTION_FLT_OVERFLOW"; break;
    case EXCEPTION_FLT_STACK_CHECK:          msg = "EXCEPTION_FLT_STACK_CHECK"; break;
    case EXCEPTION_FLT_UNDERFLOW:            msg = "EXCEPTION_FLT_UNDERFLOW"; break;
    case EXCEPTION_INT_DIVIDE_BY_ZERO:       msg = "EXCEPTION_INT_DIVIDE_BY_ZERO"; break;
    case EXCEPTION_INT_OVERFLOW:             msg = "EXCEPTION_INT_OVERFLOW"; break;
    case EXCEPTION_PRIV_INSTRUCTION:         msg = "EXCEPTION_PRIV_INSTRUCTION"; break;
    case EXCEPTION_IN_PAGE_ERROR:            msg = "EXCEPTION_IN_PAGE_ERROR"; break;
    case EXCEPTION_ILLEGAL_INSTRUCTION:      msg = "EXCEPTION_ILLEGAL_INSTRUCTION"; break;
    case EXCEPTION_NONCONTINUABLE_EXCEPTION: msg = "EXCEPTION_NONCONTINUABLE_EXCEPTION"; break;
    case EXCEPTION_STACK_OVERFLOW:           msg = "EXCEPTION_STACK_OVERFLOW"; break;
    case EXCEPTION_INVALID_DISPOSITION:      msg = "EXCEPTION_INVALID_DISPOSITION"; break;
    case EXCEPTION_GUARD_PAGE:               msg = "EXCEPTION_GUARD_PAGE"; break;
    case EXCEPTION_INVALID_HANDLE:           msg = "EXCEPTION_INVALID_HANDLE"; break;
    default:                                 msg = "UNKNOWN EXCEPTION";
  }
  printf("******************************************************\n");
  printf("%s 0x%x\n", msg, exception);
  printf("Code: 0x%x\n", pinfo->ExceptionRecord->ExceptionCode);
  printf("Flags: %s\n", (pinfo->ExceptionRecord->ExceptionFlags & EXCEPTION_NONCONTINUABLE) ? "Not-Continuable" : "Continuable");
  printf("Exception Record: 0x%p\n", pinfo->ExceptionRecord->ExceptionRecord);
  printf("Address: 0x%p\n", pinfo->ExceptionRecord->ExceptionAddress);
  printf("Number of Parameters: %d\n", pinfo->ExceptionRecord->NumberParameters);
  if(exception == (int)EXCEPTION_ACCESS_VIOLATION)
  {
    printf("------------------------------------------------------\n");
    if(pinfo->ExceptionRecord->ExceptionInformation[0] == 0)
      printf("Attempt to read inaccessible data 0x%x\n", pinfo->ExceptionRecord->ExceptionInformation[1]);
    else if(pinfo->ExceptionRecord->ExceptionInformation[0] == 1)
      printf("Attempt to write inaccessible data at 0x%x\n", pinfo->ExceptionRecord->ExceptionInformation[1]);
  }
  printf("******************************************************");
  return EXCEPTION_EXECUTE_HANDLER;
}